@@ -25,7 +25,6 @@
 #include "include/ieee80211.c"
 #include "include/strings.c"
 #include "include/byteops.c"
-#include "include/version.h"
 
 #define ARCH_INDEX(x)	((unsigned int)(unsigned char)(x))
 
@@ -38,6 +37,8 @@ static uint64_t timestamp;
 static int mybeaconsequence;
 static int myaponlinetime;
 static uint8_t myapchannel;
+static unsigned long long int pmkideapolcapwritten;
+static unsigned long long int pmkideapolcapskipped;
 static unsigned long long int pmkcapwritten;
 static unsigned long long int pmkcapskipped;
 static unsigned long long int hccapxcapwritten;
@@ -103,30 +104,25 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = M1WPA1DATA_SIZE;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
 memcpy(&packetout[PCAPREC_SIZE], &m1wpa1data, M1WPA1DATA_SIZE);
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 memcpy(mach->addr1, macsta, 6);
 memcpy(mach->addr2, macap, 6);
 memcpy(mach->addr3, macap, 6);
-
 packetout[PCAPREC_SIZE +0x22] = keyversion;
-
 wpak = (wpakey_t*)(packetout +PCAPREC_SIZE +0x26);
 #ifdef BIG_ENDIAN_HOST
 rc = byte_swap_64(rc);
 #endif
 wpak->keylen = keylen;
 wpak->replaycount = rc;
-
 for(c = 0; c < 32; c++)
 	{
 	packetout[PCAPREC_SIZE +0x33 +c] = anonce[c];
 	}
-
 if(write(fd_cap, packetout, PCAPREC_SIZE +M1WPA1DATA_SIZE) < 0)
 	{
-	perror("\nfailed to write beacon packet");
+	perror("\nfailed to write EAPOL packet");
 	}
 return;
 }
@@ -173,24 +169,20 @@ mach = (mac_t*)(packetout +PCAPREC_SIZE);
 memcpy(mach->addr1, macsta, 6);
 memcpy(mach->addr2, macap, 6);
 memcpy(mach->addr3, macap, 6);
-
 packetout[PCAPREC_SIZE +0x22] = keyversion;
-
 wpak = (wpakey_t*)(packetout +PCAPREC_SIZE +0x26);
 #ifdef BIG_ENDIAN_HOST
 rc = byte_swap_64(rc);
 #endif
 wpak->keylen = keylen;
 wpak->replaycount = rc;
-
 for(c = 0; c < 32; c++)
 	{
 	packetout[PCAPREC_SIZE +0x33 +c] = anonce[c];
 	}
-
 if(write(fd_cap, packetout, PCAPREC_SIZE +M1WPA2DATA_SIZE) < 0)
 	{
-	perror("\nfailed to write beacon packet");
+	perror("\nfailed to write EAPOL packet");
 	}
 return;
 }
@@ -231,37 +223,31 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = M1WPA2KEYVER3DATA_SIZE;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
 memcpy(&packetout[PCAPREC_SIZE], &m1wpa2keyver3data, M1WPA2KEYVER3DATA_SIZE);
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 memcpy(mach->addr1, macsta, 6);
 memcpy(mach->addr2, macap, 6);
 memcpy(mach->addr3, macap, 6);
-
 packetout[PCAPREC_SIZE +0x22] = keyversion;
-
 wpak = (wpakey_t*)(packetout +PCAPREC_SIZE +0x26);
 #ifdef BIG_ENDIAN_HOST
 rc = byte_swap_64(rc);
 #endif
 wpak->keylen = keylen;
 wpak->replaycount = rc;
-
 for(c = 0; c < 32; c++)
 	{
 	packetout[PCAPREC_SIZE +0x33 +c] = anonce[c];
 	}
-
 if(write(fd_cap, packetout, PCAPREC_SIZE +M1WPA2KEYVER3DATA_SIZE) < 0)
 	{
-	perror("\nfailed to write beacon packet");
+	perror("\nfailed to write EAPOL packet");
 	}
 return;
 }
 /*===========================================================================*/
 static void writecappmkidwpa2(int fd_cap, uint8_t *macsta, uint8_t *macap, uint8_t *pmkid)
 {
-static int c;
 static pcaprec_hdr_t *pcaph;
 static mac_t *mach;
 static const uint8_t pmkiddata[] =
@@ -274,8 +260,8 @@ static const uint8_t pmkiddata[] =
 0xaa, 0xaa, 0x03, 0x00, 0x00, 0x00, 0x88, 0x8e,
 0x02, 0x03, 0x00, 0x75, 0x02,
 0x00, 0x8a, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-0xed, 0x57, 0x5c, 0x4b, 0xca, 0xa7, 0x7a, 0xf1, 0x9e, 0x32, 0x94, 0x32, 0x63, 0x91, 0xad, 0x7d,
-0x9c, 0xbc, 0x6a, 0xb4, 0xad, 0x04, 0xf1, 0x23, 0x80, 0xb4, 0x44, 0xbe, 0xb5, 0x8d, 0x2a, 0xdd,
+0x91, 0x1b, 0xd9, 0xcb, 0x88, 0x9a, 0xcf, 0x5b, 0x0f, 0x62, 0xbf, 0x0d, 0x87, 0x6c, 0x33, 0x30,
+0x8a, 0xbf, 0x58, 0x8b, 0xc3, 0x80, 0x4c, 0x15, 0xc0, 0x42, 0x3c, 0xf2, 0x4c, 0x07, 0xa7, 0x25,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -295,22 +281,15 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = PMKIDDATA_SIZE;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
 memcpy(&packetout[PCAPREC_SIZE], &pmkiddata, PMKIDDATA_SIZE);
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 memcpy(mach->addr1, macsta, 6);
 memcpy(mach->addr2, macap, 6);
 memcpy(mach->addr3, macap, 6);
-
-for(c = 0; c < 32; c++)
-	{
-	packetout[PCAPREC_SIZE +0x33 +c] = rand() %0xff;
-	}
 memcpy(&packetout[PCAPREC_SIZE +0x8b], pmkid, 16);
-
 if(write(fd_cap, packetout, PCAPREC_SIZE +PMKIDDATA_SIZE) < 0)
 	{
-	perror("\nfailed to write beacon packet");
+	perror("\nfailed to write EAPOL packet");
 	}
 return;
 }
@@ -340,8 +319,6 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = M2DATA_SIZE +eapollen;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
-
 memcpy(&packetout[PCAPREC_SIZE], &m2data, M2DATA_SIZE);
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 memcpy(mach->addr1, macap, 6);
@@ -349,10 +326,9 @@ memcpy(mach->addr2, macsta, 6);
 memcpy(mach->addr3, macap, 6);
 memcpy(&packetout[PCAPREC_SIZE +M2DATA_SIZE], eapol, eapollen);
 memcpy(&packetout[PCAPREC_SIZE +M2DATA_SIZE +0x51], mic, 16);
-
 if(write(fd_cap, packetout, PCAPREC_SIZE +M2DATA_SIZE +eapollen) < 0)
 	{
-	perror("\nfailed to write beacon packet");
+	perror("\nfailed to write EAPOL packet");
 	}
 return;
 }
@@ -368,7 +344,9 @@ static const uint8_t beacondata[] =
 0x01, 0x08, 0x82, 0x84, 0x8b, 0x96, 0x8c, 0x12, 0x98, 0x24,
 0x03, 0x01, 0x06,
 0xdd, 0x16, 0x00, 0x50, 0xf2, 0x01, 0x01, 0x00, 0x00, 0x50, 0xf2, 0x04, 0x01, 0x00, 0x00, 0x50,
-0xf2, 0x04, 0x01, 0x00, 0x00, 0x50, 0xf2, 0x02
+0xf2, 0x04, 0x01, 0x00, 0x00, 0x50, 0xf2, 0x02,
+0xdd, 0x17, 0xff, 0xff, 0xff,
+0x00, 0xd9, 0x20, 0x21, 0x9b, 0x9b, 0x6a, 0xc9, 0x59, 0x49, 0x42, 0xe6, 0x55, 0x6a, 0x06, 0xa3, 0x23, 0x94, 0x2d, 0x94
 };
 #define BEACONDATA_SIZE sizeof(beacondata)
 
@@ -381,7 +359,6 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2 +essidlen +BEACONDATA_SIZE;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 mach->type = IEEE80211_FTYPE_MGMT;
 mach->subtype = IEEE80211_STYPE_BEACON;
@@ -393,12 +370,10 @@ if(mybeaconsequence >= 4096)
 	{
 	mybeaconsequence = 0;
 	}
-
 capap = (capap_t*)(packetout +PCAPREC_SIZE +MAC_SIZE_NORM);
 capap->timestamp = myaponlinetime++;
 capap->beaconintervall = 0x64;
 capap->capabilities = 0x431;
-
 packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +1] = essidlen;
 memcpy(&packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2], essid, essidlen);
 memcpy(&packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2 +essidlen], &beacondata, BEACONDATA_SIZE);
@@ -421,7 +396,9 @@ static const uint8_t beacondata[] =
 0x01, 0x08, 0x82, 0x84, 0x8b, 0x96, 0x8c, 0x12, 0x98, 0x24,
 0x03, 0x01, 0x06,
 0x30, 0x14, 0x01, 0x00, 0x00, 0x0f, 0xac, 0x04, 0x01, 0x00, 0x00, 0x0f, 0xac, 0x04, 0x01, 0x00,
-0x00, 0x0f, 0xac, 0x02, 0x0c, 0x00
+0x00, 0x0f, 0xac, 0x02, 0x0c, 0x00,
+0xdd, 0x17, 0xff, 0xff, 0xff,
+0x00, 0xd9, 0x20, 0x21, 0x9b, 0x9b, 0x6a, 0xc9, 0x59, 0x49, 0x42, 0xe6, 0x55, 0x6a, 0x06, 0xa3, 0x23, 0x94, 0x2d, 0x94
 };
 #define BEACONDATA_SIZE sizeof(beacondata)
 
@@ -434,7 +411,6 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2 +essidlen +BEACONDATA_SIZE;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 mach->type = IEEE80211_FTYPE_MGMT;
 mach->subtype = IEEE80211_STYPE_BEACON;
@@ -446,12 +422,10 @@ if(mybeaconsequence >= 4096)
 	{
 	mybeaconsequence = 0;
 	}
-
 capap = (capap_t*)(packetout +PCAPREC_SIZE +MAC_SIZE_NORM);
 capap->timestamp = myaponlinetime++;
 capap->beaconintervall = 0x64;
 capap->capabilities = 0x431;
-
 packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +1] = essidlen;
 memcpy(&packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2], essid, essidlen);
 memcpy(&packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2 +essidlen], &beacondata, BEACONDATA_SIZE);
@@ -474,7 +448,9 @@ static const uint8_t beacondata[] =
 0x01, 0x08, 0x82, 0x84, 0x8b, 0x96, 0x8c, 0x12, 0x98, 0x24,
 0x03, 0x01, 0x06,
 0x30, 0x14, 0x01, 0x00, 0x00, 0x0f, 0xac, 0x04, 0x01, 0x00, 0x00, 0x0f, 0xac, 0x04, 0x01, 0x00,
-0x00, 0x0f, 0xac, 0x06, 0xcc, 0x00
+0x00, 0x0f, 0xac, 0x06, 0xcc, 0x00,
+0xdd, 0x17, 0xff, 0xff, 0xff,
+0x00, 0xd9, 0x20, 0x21, 0x9b, 0x9b, 0x6a, 0xc9, 0x59, 0x49, 0x42, 0xe6, 0x55, 0x6a, 0x06, 0xa3, 0x23, 0x94, 0x2d, 0x94
 };
 #define BEACONDATA_SIZE sizeof(beacondata)
 
@@ -487,7 +463,6 @@ pcaph->ts_usec = timestamp %1000000;
 pcaph->incl_len = MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2 +essidlen +BEACONDATA_SIZE;
 pcaph->orig_len = pcaph->incl_len;
 timestamp++;
-
 mach = (mac_t*)(packetout +PCAPREC_SIZE);
 mach->type = IEEE80211_FTYPE_MGMT;
 mach->subtype = IEEE80211_STYPE_BEACON;
@@ -499,12 +474,10 @@ if(mybeaconsequence >= 4096)
 	{
 	mybeaconsequence = 0;
 	}
-
 capap = (capap_t*)(packetout +PCAPREC_SIZE +MAC_SIZE_NORM);
 capap->timestamp = myaponlinetime++;
 capap->beaconintervall = 0x64;
 capap->capabilities = 0x431;
-
 packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +1] = essidlen;
 memcpy(&packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2], essid, essidlen);
 memcpy(&packetout[PCAPREC_SIZE +MAC_SIZE_NORM +CAPABILITIESAP_SIZE +2 +essidlen], &beacondata, BEACONDATA_SIZE);
@@ -552,6 +525,267 @@ len = strlen(buffptr);
 len = chop(buffptr, len);
 return len;
 }
+static uint16_t getfield(char *lineptr, size_t bufflen, uint8_t *buff)
+{
+static size_t p;
+static uint8_t idx0;
+static uint8_t idx1;
+
+static const uint8_t hashmap[] =
+{
+0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, // 01234567
+0x08, 0x09, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 89:;<=>?
+0x00, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x00, // @ABCDEFG
+0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // HIJKLMNO
+0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // PQRSTUVW
+0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // XYZ[\]^_
+0x00, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x00, // `abcdefg
+0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // hijklmno
+};
+
+memset(buff, 0, bufflen);
+p = 0;
+while((lineptr[p] != '*') && (lineptr[p] != 0) && (p /2 <= bufflen))
+	{
+	if(! isxdigit(lineptr[p +0])) return 0;
+	if(! isxdigit(lineptr[p +1])) return 0;
+	if((lineptr[p +1] == '*') && (lineptr[p +1] == 0)) return 0;
+	idx0 = ((uint8_t)lineptr[p +0] &0x1F) ^0x10;
+	idx1 = ((uint8_t)lineptr[p +1] &0x1F) ^0x10;
+	buff[p /2] = (uint8_t)(hashmap[idx0] <<4) | hashmap[idx1];
+	p += 2;
+	if((p /2) > PMKIDEAPOL_BUFFER_LEN) return 0;
+	}
+return p /2;
+}
+/*===========================================================================*/
+static void processpmkideapolfile(char *pmkideapolname, int fd_cap)
+{
+static int len;
+static int oflen;
+static int aktread = 1;
+static FILE *fhpmkideapol;
+static uint16_t essidlen;
+static uint16_t noncelen;
+static uint16_t eapauthlen;
+static uint16_t mplen;
+static int fd_singlecap;
+static eapauth_t *eapa;
+static wpakey_t *wpak;
+static uint16_t keyinfo;
+static uint8_t keyver;
+static uint64_t rc;
+
+static const char wpa01[] = { "WPA*01*" };
+static const char wpa02[] = { "WPA*02*" };
+
+static char linein[PMKIDEAPOL_LINE_LEN +1];
+static uint8_t buffer[PMKIDEAPOL_LINE_LEN +1];
+static uint8_t macap[6];
+static uint8_t macsta[6];
+static uint8_t hash[16];
+static uint8_t anonce[32];
+static uint8_t eapol[EAPOL_AUTHLEN_MAX];
+static uint8_t essid[ESSID_LEN_MAX];
+static char singlecapname[PATH_MAX +2];
+
+if((fhpmkideapol = fopen(pmkideapolname, "r")) == NULL)
+	{
+	fprintf(stderr, "opening hash file failed %s\n", pmkideapolname);
+	return;
+	}
+while(1)
+	{
+	if((len = fgetline(fhpmkideapol, PMKIDEAPOL_LINE_LEN, linein)) == -1) break;
+	aktread++;
+	if(len < 68)
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	if((memcmp(&linein, &wpa01, 7) != 0) && (memcmp(&linein, &wpa02, 7) != 0))
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	if((linein[39] != '*') && (linein[52] != '*') && (linein[65] != '*'))
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	if(getfield(&linein[7], PMKIDEAPOL_LINE_LEN, buffer) != 16)
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	memcpy(&hash, &buffer, 16);
+
+	if(getfield(&linein[40], PMKIDEAPOL_LINE_LEN, buffer) != 6)
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	memcpy(&macap, &buffer, 6);
+	if(getfield(&linein[53], PMKIDEAPOL_LINE_LEN, buffer) != 6)
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	memcpy(&macsta, &buffer, 6);
+	essidlen = getfield(&linein[66], PMKIDEAPOL_LINE_LEN, buffer);
+	if(essidlen > 32)
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	memcpy(&essid, &buffer, essidlen);
+	if(memcmp(&linein, &wpa01, 7) == 0)
+		{
+		if(fd_cap == 0)
+			{
+			snprintf(singlecapname, 18, "%02x%02x%02x%02x%02x%02x.cap", macsta[0], macsta[1], macsta[2], macsta[3], macsta[4], macsta[5]);
+			fd_singlecap = hcxopencapdump(singlecapname);
+			if(fd_singlecap == -1)
+				{
+				fprintf(stderr, "could not create cap file\n");
+				exit(EXIT_FAILURE);
+				}
+			writecapbeaconwpa2(fd_singlecap, macap, essidlen, essid);
+			writecappmkidwpa2(fd_singlecap, macsta, macap, hash);
+			pmkideapolcapwritten++;
+			close(fd_singlecap);
+			}
+		else
+			{
+			writecapbeaconwpa2(fd_cap, macap, essidlen, essid);
+			writecappmkidwpa2(fd_cap, macsta, macap, hash);
+			pmkideapolcapwritten++;
+			}
+		}
+	else if(memcmp(&linein, &wpa02, 7) == 0)
+		{
+		oflen = 66 +essidlen *2 +1;
+		noncelen = getfield(&linein[oflen], PMKIDEAPOL_LINE_LEN, buffer);
+		if(noncelen > 32)
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		memcpy(&anonce, &buffer, 32);
+		oflen += 65;
+		eapauthlen = getfield(&linein[oflen], PMKIDEAPOL_LINE_LEN, buffer);
+		if(eapauthlen > EAPOL_AUTHLEN_MAX)
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		memcpy(&eapol, &buffer, eapauthlen);
+		oflen += eapauthlen *2 +1;
+		mplen = getfield(&linein[oflen], PMKIDEAPOL_LINE_LEN, buffer);
+		if(mplen > 1)
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		eapa = (eapauth_t*)eapol;
+		if(eapa->type != EAPOL_KEY)
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		if(eapauthlen != ntohs(eapa->len) +4)
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		wpak = (wpakey_t*)(eapol +EAPAUTH_SIZE);
+		keyver = ntohs(wpak->keyinfo) & WPA_KEY_INFO_TYPE_MASK;
+		if((keyver == 0) || (keyver > 3))
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		keyinfo = (getkeyinfo(ntohs(wpak->keyinfo)));
+		if(keyinfo == 3)
+			{
+			pmkideapolcapskipped++;
+			continue;
+			}
+		rc = wpak->replaycount;
+		#ifdef BIG_ENDIAN_HOST
+		rc = byte_swap_64(rc);
+		#endif
+		if(keyinfo == 4)
+			{
+			rc--;
+			}
+
+
+		if(fd_cap == 0)
+			{
+			snprintf(singlecapname, 18, "%02x%02x%02x%02x%02x%02x.cap", macsta[0], macsta[1], macsta[2], macsta[3], macsta[4], macsta[5]);
+			fd_singlecap = hcxopencapdump(singlecapname);
+			if(fd_singlecap == -1)
+				{
+				fprintf(stderr, "could not create cap file\n");
+				exit(EXIT_FAILURE);
+				}
+			if(keyver == 1)
+				{
+				writecapbeaconwpa1(fd_singlecap, macap, essidlen, essid);
+				writecapm1wpa1(fd_singlecap, macsta, macap, anonce, eapa->version, wpak->keylen, rc);
+				writecapm2(fd_singlecap, macsta, macap, eapauthlen, eapol, hash);
+				pmkideapolcapwritten++;
+				}
+			else if(keyver == 2)
+				{
+				writecapbeaconwpa2(fd_singlecap, macap, essidlen, essid);
+				writecapm1wpa2(fd_singlecap, macsta, macap, anonce, eapa->version, wpak->keylen, rc);
+				writecapm2(fd_singlecap, macsta, macap, eapauthlen, eapol, hash);
+				pmkideapolcapwritten++;
+				}
+			else if(keyver == 3)
+				{
+				writecapbeaconwpa2keyver3(fd_singlecap, macap, essidlen, essid);
+				writecapm1wpa2keyver3(fd_singlecap, macsta, macap, anonce, eapa->version, wpak->keylen, rc);
+				writecapm2(fd_singlecap, macsta, macap, eapauthlen, eapol, hash);
+				pmkideapolcapwritten++;
+				}
+			close(fd_singlecap);
+			}
+		else
+			{
+			if(keyver == 1)
+				{
+				writecapbeaconwpa1(fd_singlecap, macap, essidlen, essid);
+				writecapm1wpa1(fd_cap, macsta, macap, anonce, eapa->version, wpak->keylen, rc);
+				writecapm2(fd_cap, macsta, macap, eapauthlen, eapol, hash);
+				pmkideapolcapwritten++;
+				}
+			else if(keyver == 2)
+				{
+				writecapbeaconwpa2(fd_singlecap, macap, essidlen, essid);
+				writecapm1wpa2(fd_cap, macsta, macap, anonce, eapa->version, wpak->keylen, rc);
+				writecapm2(fd_cap, macsta, macap, eapauthlen, eapol, hash);
+				pmkideapolcapwritten++;
+				}
+			else if(keyver == 3)
+				{
+				writecapbeaconwpa2keyver3(fd_singlecap, macap, essidlen, essid);
+				writecapm1wpa2keyver3(fd_cap, macsta, macap, anonce, eapa->version, wpak->keylen, rc);
+				writecapm2(fd_cap, macsta, macap, eapauthlen, eapol, hash);
+				pmkideapolcapwritten++;
+				}
+			}
+		}
+	else
+		{
+		pmkideapolcapskipped++;
+		continue;
+		}
+	}
+return;
+}
 /*===========================================================================*/
 static void processpmkidfile(char *pmkidname, int fd_cap)
 {
@@ -566,7 +800,6 @@ static uint8_t macap[6];
 static uint8_t macsta[6];
 static uint8_t pmkid[16];
 static uint8_t essid[ESSID_LEN_MAX];
-
 static char singlecapname[PATH_MAX +2];
 
 if((fhpmkid = fopen(pmkidname, "r")) == NULL)
@@ -574,7 +807,6 @@ if((fhpmkid = fopen(pmkidname, "r")) == NULL)
 	fprintf(stderr, "opening hash file failed %s\n", pmkidname);
 	return;
 	}
-
 while(1)
 	{
 	if((len = fgetline(fhpmkid, PMKID_LINE_LEN, linein)) == -1)
@@ -616,21 +848,18 @@ while(1)
 		pmkcapskipped++;
 		continue;
 		}
-
 	if(hex2bin(&linein[33], macap, 6) != true)
 		{
 		fprintf(stderr, "reading hash line %d failed: %s\n", aktread, linein);
 		pmkcapskipped++;
 		continue;
 		}
-
 	if(hex2bin(&linein[46], macsta, 6) != true)
 		{
 		fprintf(stderr, "reading hash line %d failed: %s\n", aktread, linein);
 		pmkcapskipped++;
 		continue;
 		}
-
 	if(hex2bin(&linein[59], essid, essidlen/2) != true)
 		{
 		fprintf(stderr, "reading hash line %d failed: %s\n", aktread, linein);
@@ -669,7 +898,6 @@ static struct stat statinfo;
 static hccapx_t *hcxptr;
 static int fd_singlecap;
 static FILE *fhhcx;
-
 static eapauth_t *eapa;
 static wpakey_t *wpak;
 static uint16_t keyinfo;
@@ -684,19 +912,16 @@ if(stat(hccapxname, &statinfo) != 0)
 	fprintf(stderr, "can't stat %s\n", hccapxname);
 	return;
 	}
-
 if((statinfo.st_size %HCCAPX_SIZE) != 0)
 	{
 	fprintf(stderr, "file corrupt\n");
 	return;
 	}
-
 if((fhhcx = fopen(hccapxname, "rb")) == NULL)
 	{
 	fprintf(stderr, "error opening file %s", hccapxname);
 	return;
 	}
-
 hcxptr = (hccapx_t*)hcxdata;
 while(fread(&hcxdata, HCCAPX_SIZE, 1, fhhcx) == 1)
 	{
@@ -726,7 +951,6 @@ while(fread(&hcxdata, HCCAPX_SIZE, 1, fhhcx) == 1)
 		hccapxcapskipped++;
 		continue;
 		}
-
 	wpak = (wpakey_t*)(hcxptr->eapol +EAPAUTH_SIZE);
 	keyver = ntohs(wpak->keyinfo) & WPA_KEY_INFO_TYPE_MASK;
 	if(keyver != hcxptr->keyver)
@@ -1314,7 +1538,7 @@ return;
 __attribute__ ((noreturn))
 static void version(char *eigenname)
 {
-printf("%s %s (C) %s ZeroBeat\n", eigenname, VERSION, VERSION_JAHR);
+printf("%s %s (C) %s ZeroBeat\n", eigenname, VERSION_TAG, VERSION_YEAR);
 exit(EXIT_SUCCESS);
 }
 /*---------------------------------------------------------------------------*/
@@ -1332,13 +1556,14 @@ printf("%s %s (C) %s ZeroBeat\n"
 	"-h        : show this help\n"
 	"-v        : show version\n"
 	"\n"
-	"--pmkid=<file>  : input PMKID hash file\n"
-	"--hccapx=<file> : input hashcat hccapx file\n"
-	"--hccap=<file>  : input hashcat hccap file\n"
-	"--john=<file>   : input John the Ripper WPAPSK hash file\n"
-	"--help          : show this help\n"
-	"--version       : show version\n"
-	"\n", eigenname, VERSION, VERSION_JAHR, eigenname);
+	"--pmkid-eapol=<file> : input PMKID EAPOL combi hash file\n"
+	"--pmkid=<file>       : input PMKID hash file\n"
+	"--hccapx=<file>      : input hashcat hccapx file\n"
+	"--hccap=<file>       : input hashcat hccap file\n"
+	"--john=<file>        : input John the Ripper WPAPSK hash file\n"
+	"--help               : show this help\n"
+	"--version            : show version\n"
+	"\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
 exit(EXIT_SUCCESS);
 }
 /*---------------------------------------------------------------------------*/
@@ -1346,7 +1571,7 @@ __attribute__ ((noreturn))
 static void usageerror(char *eigenname)
 {
 printf("%s %s (C) %s by ZeroBeat\n"
-	"usage: %s -h for help\n", eigenname, VERSION, VERSION_JAHR, eigenname);
+	"usage: %s -h for help\n", eigenname, VERSION_TAG, VERSION_YEAR, eigenname);
 exit(EXIT_FAILURE);
 }
 /*===========================================================================*/
@@ -1355,6 +1580,7 @@ int main(int argc, char *argv[])
 static int auswahl;
 static int index;
 static int fd_cap = 0;
+static char *pmkideapolname = NULL;
 static char *pmkidname = NULL;
 static char *hccapxname = NULL;
 static char *hccapname = NULL;
@@ -1364,6 +1590,7 @@ static char *capname = NULL;
 static const char *short_options = "c:hv";
 static const struct option long_options[] =
 {
+	{"pmkid-eapol",			required_argument,	NULL,	HCXP_PMKID_EAPOL},
 	{"pmkid",			required_argument,	NULL,	HCXP_PMKID},
 	{"hccapx",			required_argument,	NULL,	HCXP_HCCAPX},
 	{"hccap",			required_argument,	NULL,	HCXP_HCCAP},
@@ -1382,6 +1609,10 @@ while((auswahl = getopt_long (argc, argv, short_options, long_options, &index))
 	{
 	switch (auswahl)
 		{
+		case HCXP_PMKID_EAPOL:
+		pmkideapolname = optarg;
+		break;
+
 		case HCXP_PMKID:
 		pmkidname = optarg;
 		break;
@@ -1434,6 +1665,11 @@ if(capname != NULL)
 		}
 	}
 
+if(pmkideapolname != NULL)
+	{
+	processpmkideapolfile(pmkideapolname, fd_cap);
+	}
+
 if(pmkidname != NULL)
 	{
 	processpmkidfile(pmkidname, fd_cap);
@@ -1460,7 +1696,10 @@ if(fd_cap != 0)
 	close(fd_cap);
 	removeemptycap(capname);
 	}
-
+if(pmkideapolcapwritten > 0)
+	{
+	fprintf(stdout, "PMKIDs/EAPOL messages written to capfile(s): %llu (%llu skipped)\n", pmkideapolcapwritten, pmkideapolcapskipped);
+	}
 if(pmkcapwritten > 0)
 	{
 	fprintf(stdout, "PMKIDs written to capfile(s): %llu (%llu skipped)\n", pmkcapwritten, pmkcapskipped);
